@@ -15,14 +15,14 @@
 #include <string.h>
 
 #include <mate-panel-applet.h>
-#include <mate-panel-applet-mateconf.h>
+#include <mate-panel-applet-gsettings.h>
 
 #include <stdlib.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libmatewnck/libmatewnck.h>
-#include <mateconf/mateconf-client.h>
+#include <gio/gio.h>
 
 #include "workspace-switcher.h"
 
@@ -32,9 +32,13 @@
 #define MAX_REASONABLE_ROWS 16
 #define DEFAULT_ROWS 1
 
+#define WORKSPACE_SWITCHER_SCHEMA "org.mate.panel.applet.workspace-switcher"
+
 #define NEVER_SENSITIVE "never_sensitive"
-#define NUM_WORKSPACES "/apps/marco/general/num_workspaces"
-#define WORKSPACE_NAME "/apps/marco/workspace_names/name_1"
+#define MARCO_GENERAL_SCHEMA "org.mate.Marco.general"
+#define NUM_WORKSPACES "num-workspaces"
+#define MARCO_WORSKACES_SCHEMA "org.mate.Marco.workspace-names"
+#define WORKSPACE_NAME "name-1"
 
 #define WORKSPACE_SWITCHER_ICON "mate-panel-workspace-switcher"
 
@@ -58,6 +62,7 @@ typedef struct {
 	GtkWidget* workspace_names_label;
 	GtkWidget* workspace_names_scroll;
 	GtkWidget* display_workspaces_toggle;
+	GtkWidget* wrap_workspaces_toggle;
 	GtkWidget* all_workspaces_radio;
 	GtkWidget* current_only_radio;
 	GtkWidget* num_rows_spin;	       /* for vertical layout this is cols */
@@ -70,9 +75,9 @@ typedef struct {
 	int n_rows;				/* for vertical layout this is cols */
 	MatewnckPagerDisplayMode display_mode;
 	gboolean display_all;
+	gboolean wrap_workspaces;
 
-	/* mateconf listeners id */
-	guint listeners[3];
+	GSettings* settings;
 } PagerData;
 
 static void display_properties_dialog(GtkAction* action, PagerData* pager);
@@ -189,17 +194,34 @@ static void applet_change_orient(MatePanelApplet* applet, MatePanelAppletOrient
 
 static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, GdkPixmap* pixmap, PagerData* pager)
 {
+        /* taken from the TrashApplet */
+        GtkRcStyle *rc_style;
+        GtkStyle *style;
+
+        /* reset style */
+        gtk_widget_set_style (GTK_WIDGET (pager->pager), NULL);
+        rc_style = gtk_rc_style_new ();
+        gtk_widget_modify_style (GTK_WIDGET (pager->pager), rc_style);
+        g_object_unref (rc_style);
+
 	switch (type)
 	{
-		case PANEL_NO_BACKGROUND:
-			matewnck_pager_set_shadow_type(MATEWNCK_PAGER(pager->pager), GTK_SHADOW_IN);
-			break;
-		case PANEL_COLOR_BACKGROUND:
-			matewnck_pager_set_shadow_type(MATEWNCK_PAGER(pager->pager), GTK_SHADOW_NONE);
-			break;
-		case PANEL_PIXMAP_BACKGROUND:
-			matewnck_pager_set_shadow_type(MATEWNCK_PAGER(pager->pager), GTK_SHADOW_NONE);
-			break;
+                case PANEL_COLOR_BACKGROUND:
+                        gtk_widget_modify_bg (GTK_WIDGET (pager->pager), GTK_STATE_NORMAL, color);
+                        break;
+
+                case PANEL_PIXMAP_BACKGROUND:
+                        style = gtk_style_copy (gtk_widget_get_style (GTK_WIDGET (pager->pager)));
+                        if (style->bg_pixmap[GTK_STATE_NORMAL])
+                                g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+                        style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref(pixmap);
+                        gtk_widget_set_style (GTK_WIDGET (pager->pager), style);
+                        g_object_unref (style);
+                        break;
+
+                case PANEL_NO_BACKGROUND:
+                default:
+                        break;
 	}
 }
 
@@ -248,6 +270,10 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 			{
 				index += n_columns;
 			}
+			else if (pager->wrap_workspaces && index == n_workspaces - 1)
+			{
+				index = 0;
+			}
 			else if ((index < n_workspaces - 1 && index + in_last_row != n_workspaces - 1) || (index == n_workspaces - 1 && in_last_row != 0))
 			{
 				index = (index % n_columns) + 1;
@@ -256,7 +282,13 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 
 		case GDK_SCROLL_RIGHT:
 			if (index < n_workspaces - 1)
+			{
 				index++;
+			}
+			else if (pager->wrap_workspaces)
+			{
+			        index = 0;
+			}
 			break;
 
 		case GDK_SCROLL_UP:
@@ -268,6 +300,10 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 			{
 				index = ((pager->n_rows - 1) * n_columns) + (index % n_columns) - 1;
 			}
+			else if (pager->wrap_workspaces)
+			{
+				index = n_workspaces - 1;
+			}
 
 			if (index >= n_workspaces)
 				index -= n_columns;
@@ -275,7 +311,13 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 
 		case GDK_SCROLL_LEFT:
 			if (index > 0)
+			{
 				index--;
+			}
+			else if (pager->wrap_workspaces)
+			{
+				index = n_workspaces - 1;
+			}
 			break;
 		default:
 			g_assert_not_reached();
@@ -289,17 +331,7 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 
 static void destroy_pager(GtkWidget* widget, PagerData* pager)
 {
-	MateConfClient* client = mateconf_client_get_default();
-
-	mateconf_client_notify_remove(client, pager->listeners[0]);
-	mateconf_client_notify_remove(client, pager->listeners[1]);
-	mateconf_client_notify_remove(client, pager->listeners[2]);
-
-	g_object_unref(G_OBJECT(client));
-
-	pager->listeners[0] = 0;
-	pager->listeners[1] = 0;
-	pager->listeners[2] = 0;
+	g_object_unref (pager->settings);
 
 	if (pager->properties_dialog)
 		gtk_widget_destroy(pager->properties_dialog);
@@ -334,14 +366,11 @@ static const GtkActionEntry pager_menu_actions[] = {
 	}
 };
 
-static void num_rows_changed(MateConfClient* client, guint cnxn_id, MateConfEntry* entry, PagerData* pager)
+static void num_rows_changed(GSettings* settings, gchar* key, PagerData* pager)
 {
 	int n_rows = DEFAULT_ROWS;
 
-	if (entry->value != NULL && entry->value->type == MATECONF_VALUE_INT)
-	{
-		n_rows = mateconf_value_get_int(entry->value);
-	}
+	n_rows = g_settings_get_int (settings, key);
 
 	n_rows = CLAMP(n_rows, 1, MAX_REASONABLE_ROWS);
 
@@ -352,14 +381,11 @@ static void num_rows_changed(MateConfClient* client, guint cnxn_id, MateConfEntr
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_rows_spin), pager->n_rows);
 }
 
-static void display_workspace_names_changed(MateConfClient* client, guint cnxn_id, MateConfEntry* entry, PagerData* pager)
+static void display_workspace_names_changed(GSettings* settings, gchar* key, PagerData* pager)
 {
 	gboolean value = FALSE; /* Default value */
 
-	if (entry->value != NULL && entry->value->type == MATECONF_VALUE_BOOL)
-	{
-		value = mateconf_value_get_bool(entry->value);
-	}
+	value = g_settings_get_boolean (settings, key);
 
 	if (value)
 	{
@@ -379,14 +405,11 @@ static void display_workspace_names_changed(MateConfClient* client, guint cnxn_i
 }
 
 
-static void all_workspaces_changed(MateConfClient* client, guint cnxn_id, MateConfEntry* entry, PagerData* pager)
+static void all_workspaces_changed(GSettings* settings, gchar* key, PagerData* pager)
 {
 	gboolean value = TRUE; /* Default value */
 
-	if (entry->value != NULL && entry->value->type == MATECONF_VALUE_BOOL)
-	{
-		value = mateconf_value_get_bool(entry->value);
-	}
+	value = g_settings_get_boolean (settings, key);
 
 	pager->display_all = value;
 	pager_update(pager);
@@ -410,30 +433,41 @@ static void all_workspaces_changed(MateConfClient* client, guint cnxn_id, MateCo
 	}
 }
 
-static void setup_mateconf(PagerData* pager)
+static void wrap_workspaces_changed(GSettings* settings, gchar* key, PagerData* pager)
 {
-	MateConfClient* client;
-	char* key;
+	gboolean value = FALSE; /* Default value */
 
-	client = mateconf_client_get_default();
+	value = g_settings_get_boolean (settings, key);
 
-	key = mate_panel_applet_mateconf_get_full_key(MATE_PANEL_APPLET(pager->applet), "num_rows");
-	pager->listeners[0] = mateconf_client_notify_add(client, key, (MateConfClientNotifyFunc) num_rows_changed, pager, NULL, NULL);
+	pager->wrap_workspaces = value;
 
-	g_free(key);
+	if (pager->wrap_workspaces_toggle && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pager->wrap_workspaces_toggle)) != value)
+	{
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pager->wrap_workspaces_toggle), value);
+	}
+}
 
+static void setup_gsettings(PagerData* pager)
+{
+	pager->settings = mate_panel_applet_settings_new (MATE_PANEL_APPLET (pager->applet), WORKSPACE_SWITCHER_SCHEMA);
 
-	key = mate_panel_applet_mateconf_get_full_key(MATE_PANEL_APPLET(pager->applet), "display_workspace_names");
-	pager->listeners[1] = mateconf_client_notify_add(client, key, (MateConfClientNotifyFunc) display_workspace_names_changed, pager, NULL, NULL);
+	g_signal_connect (pager->settings,
+					  "changed::num-rows",
+					  G_CALLBACK (num_rows_changed),
+					  pager);
+	g_signal_connect (pager->settings,
+					  "changed::display-workspace-names",
+					  G_CALLBACK (display_workspace_names_changed),
+					  pager);
+	g_signal_connect (pager->settings,
+					  "changed::display-all-workspaces",
+					  G_CALLBACK (all_workspaces_changed),
+					  pager);
+	g_signal_connect (pager->settings,
+					  "changed::wrap-workspaces",
+					  G_CALLBACK (wrap_workspaces_changed),
+					  pager);
 
-	g_free(key);
-
-	key = mate_panel_applet_mateconf_get_full_key(MATE_PANEL_APPLET(pager->applet), "display_all_workspaces");
-	pager->listeners[2] = mateconf_client_notify_add(client, key, (MateConfClientNotifyFunc) all_workspaces_changed, pager, NULL, NULL);
-
-	g_free(key);
-
-	g_object_unref(G_OBJECT(client));
 }
 
 gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
@@ -441,10 +475,8 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	PagerData* pager;
 	GtkActionGroup* action_group;
 	gchar* ui_path;
-	GError* error;
 	gboolean display_names;
-
-	mate_panel_applet_add_preferences(applet, "/schemas/apps/workspace_switcher_applet/prefs", NULL);
+	gboolean wrap_workspaces;
 
 	pager = g_new0(PagerData, 1);
 
@@ -452,29 +484,15 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 
 	mate_panel_applet_set_flags(MATE_PANEL_APPLET(pager->applet), MATE_PANEL_APPLET_EXPAND_MINOR);
 
-	setup_mateconf(pager);
+	setup_gsettings(pager);
 
-	error = NULL;
-	pager->n_rows = mate_panel_applet_mateconf_get_int(applet, "num_rows", &error);
-
-	if (error)
-	{
-		g_printerr(_("Error loading num_rows value for Workspace Switcher: %s\n"), error->message);
-		g_error_free(error);
-		/* leave current value */
-	}
+	pager->n_rows = g_settings_get_int(pager->settings, "num-rows");
 
 	pager->n_rows = CLAMP(pager->n_rows, 1, MAX_REASONABLE_ROWS);
 
-	error = NULL;
-	display_names = mate_panel_applet_mateconf_get_bool(applet, "display_workspace_names", &error);
+	display_names = g_settings_get_boolean(pager->settings, "display-workspace-names");
 
-	if (error)
-	{
-		g_printerr(_("Error loading display_workspace_names value for Workspace Switcher: %s\n"), error->message);
-		g_error_free(error);
-		/* leave current value */
-	}
+	wrap_workspaces = g_settings_get_boolean(pager->settings, "wrap-workspaces");
 
 	if (display_names)
 	{
@@ -485,15 +503,7 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 		pager->display_mode = MATEWNCK_PAGER_DISPLAY_CONTENT;
 	}
 
-	error = NULL;
-	pager->display_all = mate_panel_applet_mateconf_get_bool(applet, "display_all_workspaces", &error);
-
-	if (error)
-	{
-		g_printerr(_("Error loading display_all_workspaces value for Workspace Switcher: %s\n"), error->message);
-		g_error_free(error);
-		/* leave current value */
-	}
+	pager->display_all = g_settings_get_boolean(pager->settings, "display-all-workspaces");
 
 	switch (mate_panel_applet_get_orient(applet))
 	{
@@ -588,20 +598,24 @@ static void display_about_dialog(GtkAction* action, PagerData* pager)
 		NULL);
 }
 
-
-static void display_workspace_names_toggled(GtkToggleButton*button, PagerData* pager)
+static void wrap_workspaces_toggled(GtkToggleButton* button, PagerData* pager)
 {
-	mate_panel_applet_mateconf_set_bool(MATE_PANEL_APPLET(pager->applet), "display_workspace_names", gtk_toggle_button_get_active(button), NULL);
+	g_settings_set_boolean(pager->settings, "wrap-workspaces", gtk_toggle_button_get_active(button));
+}
+
+static void display_workspace_names_toggled(GtkToggleButton* button, PagerData* pager)
+{
+	g_settings_set_boolean(pager->settings, "display-workspace-names", gtk_toggle_button_get_active(button));
 }
 
 static void all_workspaces_toggled(GtkToggleButton* button, PagerData* pager)
 {
-	mate_panel_applet_mateconf_set_bool(MATE_PANEL_APPLET(pager->applet), "display_all_workspaces", gtk_toggle_button_get_active(button), NULL);
+	g_settings_set_boolean(pager->settings, "display-all-workspaces", gtk_toggle_button_get_active(button));
 }
 
 static void num_rows_value_changed(GtkSpinButton* button, PagerData* pager)
 {
-	mate_panel_applet_mateconf_set_int(MATE_PANEL_APPLET(pager->applet), "num_rows", gtk_spin_button_get_value_as_int(button), NULL);
+	g_settings_set_int(pager->settings, "num-rows", gtk_spin_button_get_value_as_int(button));
 }
 
 static void update_workspaces_model(PagerData* pager)
@@ -706,6 +720,7 @@ static void properties_dialog_destroyed(GtkWidget* widget, PagerData* pager)
 	pager->workspace_names_label = NULL;
 	pager->workspace_names_scroll = NULL;
 	pager->display_workspaces_toggle = NULL;
+	pager->wrap_workspaces_toggle = NULL;
 	pager->all_workspaces_radio = NULL;
 	pager->current_only_radio = NULL;
 	pager->num_rows_spin = NULL;
@@ -752,28 +767,15 @@ static void close_dialog(GtkWidget* button, gpointer data)
 #define WID(s) GTK_WIDGET(gtk_builder_get_object(builder, s))
 
 static void
-setup_sensitivity(PagerData* pager, GtkBuilder* builder, const char* wid1, const char* wid2, const char* wid3, const char* key)
+setup_sensitivity(PagerData* pager, GtkBuilder* builder, const char* wid1, const char* wid2, const char* wid3, GSettings* settings, const char* key)
 {
-	MatePanelApplet* applet = MATE_PANEL_APPLET(pager->applet);
-	MateConfClient* client = mateconf_client_get_default();
-	char* fullkey;
 	GtkWidget* w;
 
-	if (key[0] == '/')
-		fullkey = g_strdup(key);
-	else
-		fullkey = mate_panel_applet_mateconf_get_full_key(applet, key);
-
-	if (mateconf_client_key_is_writable(client, fullkey, NULL))
+	if (g_settings_is_writable(settings, key))
 	{
-		g_object_unref(G_OBJECT(client));
-		g_free(fullkey);
 		return;
 	}
 
-	g_object_unref(G_OBJECT(client));
-	g_free(fullkey);
-
 	w = WID(wid1);
 	g_assert(w != NULL);
 	g_object_set_data(G_OBJECT(w), NEVER_SENSITIVE, GINT_TO_POINTER(1));
@@ -802,27 +804,43 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	GtkTreeViewColumn* column;
 	GtkCellRenderer* cell;
 	int nr_ws, i;
+	GSettings *marco_general_settings;
+	GSettings *marco_workspaces_settings;
+
+	marco_general_settings = g_settings_new (MARCO_GENERAL_SCHEMA);
+	marco_workspaces_settings = g_settings_new (MARCO_WORSKACES_SCHEMA);
 
 	pager->workspaces_frame = WID("workspaces_frame");
 	pager->workspace_names_label = WID("workspace_names_label");
 	pager->workspace_names_scroll = WID("workspace_names_scroll");
 
 	pager->display_workspaces_toggle = WID("workspace_name_toggle");
-	setup_sensitivity(pager, builder, "workspace_name_toggle", NULL, NULL, "display_workspace_names" /* key */);
+	setup_sensitivity(pager, builder, "workspace_name_toggle", NULL, NULL, pager->settings, "display-workspace-names" /* key */);
+
+	pager->wrap_workspaces_toggle = WID("workspace_wrap_toggle");
+	setup_sensitivity(pager, builder, "workspace_wrap_toggle", NULL, NULL, pager->settings, "wrap-workspaces" /* key */);
 
 	pager->all_workspaces_radio = WID("all_workspaces_radio");
 	pager->current_only_radio = WID("current_only_radio");
-	setup_sensitivity(pager, builder, "all_workspaces_radio", "current_only_radio", "label_row_col", "display_all_workspaces" /* key */);
+	setup_sensitivity(pager, builder, "all_workspaces_radio", "current_only_radio", "label_row_col", pager->settings, "display-all-workspaces" /* key */);
 
 	pager->num_rows_spin = WID("num_rows_spin");
 	pager->label_row_col = WID("label_row_col");
-	setup_sensitivity(pager, builder, "num_rows_spin", NULL, NULL, "num_rows" /* key */);
+	setup_sensitivity(pager, builder, "num_rows_spin", NULL, NULL, pager->settings, "num-rows" /* key */);
 
 	pager->num_workspaces_spin = WID("num_workspaces_spin");
-	setup_sensitivity(pager, builder, "num_workspaces_spin", NULL, NULL, NUM_WORKSPACES /* key */);
+	setup_sensitivity(pager, builder, "num_workspaces_spin", NULL, NULL, marco_general_settings, NUM_WORKSPACES /* key */);
 
 	pager->workspaces_tree = WID("workspaces_tree_view");
-	setup_sensitivity(pager, builder, "workspaces_tree_view", NULL, NULL, WORKSPACE_NAME /* key */);
+	setup_sensitivity(pager, builder, "workspaces_tree_view", NULL, NULL, marco_workspaces_settings, WORKSPACE_NAME /* key */);
+
+	g_object_unref (marco_general_settings);
+	g_object_unref (marco_workspaces_settings);
+
+
+	/* Wrap workspaces: */
+
+	g_signal_connect(G_OBJECT(pager->wrap_workspaces_toggle), "toggled", (GCallback) wrap_workspaces_toggled, pager);
 
 	/* Display workspace names: */
 
