@@ -20,7 +20,6 @@
 #include <libpanel-util/panel-cleanup.h>
 #include <libpanel-util/panel-glib.h>
 
-#include "panel-mateconf.h"
 #include "panel-profile.h"
 #include "panel-config-global.h"
 #include "panel-shell.h"
@@ -31,6 +30,7 @@
 #include "panel-lockdown.h"
 #include "panel-icon-names.h"
 #include "panel-reset.h"
+#include "panel-run-dialog.h"
 #include "xstuff.h"
 
 #include "nothing.cP"
@@ -42,6 +42,7 @@ GSList *panel_list = NULL;
 static char*    deprecated_profile;
 static gboolean replace = FALSE;
 static gboolean reset = FALSE;
+static gboolean run_dialog = FALSE;
 
 static const GOptionEntry options[] = {
   { "replace", 0, 0, G_OPTION_ARG_NONE, &replace, N_("Replace a currently running panel"), NULL },
@@ -49,6 +50,8 @@ static const GOptionEntry options[] = {
   { "profile", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &deprecated_profile, NULL, NULL },
   /* this feature was request in #mate irc channel */
   { "reset", 0, 0, G_OPTION_ARG_NONE, &reset, N_("Reset the panel configuration to default"), NULL },
+  /* open run dialog */
+  { "run-dialog", 0, 0, G_OPTION_ARG_NONE, &run_dialog, N_("Execute the run dialog"), NULL },
   { NULL }
 };
 
@@ -100,6 +103,21 @@ main (int argc, char **argv)
 		return 0;
 	}
 
+	/* open the run dialog and exit */
+	if (run_dialog == TRUE)
+	{
+		panel_multiscreen_init ();
+		panel_global_config_load ();
+		panel_lockdown_init ();
+		panel_profile_settings_load ();
+		panel_run_dialog_present (gdk_screen_get_default (),
+								  gtk_get_current_event_time ());
+		panel_run_dialog_quit_on_destroy ();
+		gtk_main ();
+		panel_lockdown_finalize ();
+		panel_cleanup_do ();
+		return 0;
+	}
 
 	if (!egg_get_desktop_file ()) {
 		g_set_application_name (_("Panel"));
@@ -115,11 +133,6 @@ main (int argc, char **argv)
 	panel_multiscreen_init ();
 	panel_init_stock_icons_and_items ();
 
-	mateconf_client_add_dir (panel_mateconf_get_client (),
-			      "/desktop/mate/interface",
-			      MATECONF_CLIENT_PRELOAD_NONE,
-			      NULL);
-
 	panel_global_config_load ();
 	panel_lockdown_init ();
 	panel_profile_load ();
@@ -143,10 +156,6 @@ main (int argc, char **argv)
 
 	panel_lockdown_finalize ();
 
-	mateconf_client_remove_dir (panel_mateconf_get_client (),
-				 "/desktop/mate/interface",
-				 NULL);
-
 	panel_cleanup_do ();
 
 	return 0;
